import { world, system } from "@minecraft/server";
import { handleHit, summon } from "./games/snowball_range";
import { runAvalanche, triggerAvalanche, digAvalanche } from "./games/avalanche";
import { spawnSnowballStack } from "./games/snowball_fight";
import { giveCandy, rewardPlayer } from "./candy";
import { dropItems, getDistance, getNearestPlayer, log } from "./utils";
import { endWorkshop, startWorkshop, equipPlayer } from "./games/workshop";
import { load, save } from "./storage/PlayerInventories";
import { openToolsShop, openHorseShop, openFurnitureShop1, openFurnitureShop2, openFestiveBoxShop, openFestiveTreeShop, openDecorationShop1, openDecorationShop2 } from "./shops/Shops";
import { ModalFormData } from "@minecraft/server-ui";
import { displayText, showMenu } from "./navigation_tool";
import { Show } from "./Show";

const overworld = world.getDimension("overworld");

const show = new Show();

world.afterEvents.projectileHitEntity.subscribe((data) => {
    if (data.projectile.typeId == "minecraft:snowball" && data.source != null && data.source.hasTag("playing_snowball_range")) {
        handleHit(data);
    }
});

world.afterEvents.entityHitEntity.subscribe((data) => {
    if (data.damagingEntity.typeId == "minecraft:player") {
        switch (data.hitEntity.typeId) {
            case "podcrash:holder":
                digAvalanche(data);
                break;
            case "podcrash:log":
                data.damagingEntity.runCommand("execute as @s[hasitem={item=podcrash:axe,location=slot.weapon.mainhand,quantity=1..}] run scriptevent podcrash:cut_wood");
                break;
        }
    }
});

const cooldowns = new Map();

world.afterEvents.itemCompleteUse.subscribe((data) => {
    if(data.itemStack.typeId==="podcrash:hot_chocolate") {
        data.source.addEffect("speed", 100, {amplifier:2,showParticles:false});
    }
})

world.afterEvents.itemUse.subscribe((data) => {
    const player = data.source;
    if (cooldowns.has(player.name)) {
        if (Date.now() - cooldowns.get(player.name) < 100) {
            return;
        } else {
            cooldowns.delete(player.name);
        }
    }
    switch (data.itemStack.typeId) {
        case "podcrash:furniture":
            player.runCommand("function furniture/enter_editor");
            load(player);
            cooldowns.set(player.name, Date.now());
            break;
        case "podcrash:exit_furniture":
            player.runCommand("function furniture/exit_editor");
            save(player);
            cooldowns.set(player.name, Date.now());
            break;
        case "podcrash:horse":
            if (world.scoreboard.getObjective("horse").getScore(player) > 0) {
                if (overworld.getEntities({ tags: [player.name] }).length > 0) {
                    player.runCommand(`tellraw @s { "rawtext": [ { "translate": "feature.horse.already_spawned" } ] }`);
                } else {
                    const horse = overworld.spawnEntity("minecraft:horse", player.location);
                    horse.addTag(player.name);
                    player.runCommand(`ride @s start_riding @e[type=minecraft:horse,c=1,tag="${player.name}"] teleport_rider`);
                }
            } else {
                player.runCommand(`tellraw @s { "rawtext": [ { "translate": "feature.horse.no_horse" } ] }`);
            }
            cooldowns.set(player.name, Date.now());
            break;
        case "podcrash:furniture_tool":
            const form = new ModalFormData();
            const target = overworld.getEntities({ families: ["podcrash_furniture"], closest: 1, location: player.location })[0];
            if (target === undefined) {
                player.runCommand(`tellraw @s { "rawtext": [ { "translate": "feature.furniture_tool.no_furniture" } ] }`);
                return;
            }
            let s = target.typeId.replace("podcrash:", "");
            let ssplit = s.split("_");
            s = "";
            ssplit.forEach(element => {
                s = s + element.charAt(0).toUpperCase() + element.substring(1) + " ";
            });

            //READ ROTATION
            var rot;
            var xRot = Math.round(target.getViewDirection().x), zRot = Math.round(target.getViewDirection().z);
            if (Math.abs(xRot) == Math.abs(zRot)) {
                if (Math.abs(target.getViewDirection().x) > Math.abs(target.getViewDirection().z)) zRot = 0
                else xRot = 0
            }
            if (xRot == 1 && zRot == 0) rot = 270;
            if (xRot == -1 && zRot == 0) rot = 90;
            if (xRot == 0 && zRot == 1) rot = 0;
            if (xRot == 0 && zRot == -1) rot = 180;

            form.title(`Modify ${s}`);
            form.slider(`Rotate`, 0, 360, 45, rot);
            form.slider(`Move X`, -1, 1, 0.1, 0);
            form.slider(`Move Y`, -1, 1, 0.1, 0);
            form.slider(`Move Z`, -1, 1, 0.1, 0);

            form.show(player).then(r => {
                if (r.canceled || r.formValues == null) return;
                let [rotation, x, y, z] = r.formValues;
                target.runCommand(`tp @s ~${x} ~${y} ~${z} ${rotation} ~`);
            });
            break;
        case "minecraft:compass":
            showMenu(player);
            break;
    }
});


system.afterEvents.scriptEventReceive.subscribe((data) => {
    if (data.id == "podcrash:sr_spawn") summon();
    if (data.id == "podcrash:avalanche_trigger") triggerAvalanche();
    if (data.id == "podcrash:spawn_snowball_stack") spawnSnowballStack();
    if (data.id == "podcrash:leave_snowball_fight" && data.sourceEntity != null) rewardPlayer(data.sourceEntity, "snowball_fight_hits", 2);
    if (data.id == "podcrash:finish_snowball_range" && data.sourceEntity != null) rewardPlayer(data.sourceEntity, "snowball_range_score", 5);
    if (data.id == "podcrash:give_candy" && data.sourceEntity != null) giveCandy(data.sourceEntity, data.message);
    if (data.id == "podcrash:distance_to_player" && data.sourceEntity != null) log(`d=${getDistance(data.sourceEntity.location, getNearestPlayer(data.sourceEntity.location).location)}`);
    if (data.id == "podcrash:workshop_start") startWorkshop();
    if (data.id == "podcrash:workshop_end") endWorkshop();
    if (data.id == "podcrash:workshop_equip" && data.sourceEntity != null) equipPlayer(data.sourceEntity);
    if (data.id == "podcrash:save_inventory" && data.sourceEntity != null) save(data.sourceEntity);
    if (data.id == "podcrash:load_inventory" && data.sourceEntity != null) load(data.sourceEntity);
    if (data.id == "podcrash:open_tools_shop" && data.sourceEntity != null) openToolsShop(data.sourceEntity);
    if (data.id == "podcrash:open_horse_shop" && data.sourceEntity != null) openHorseShop(data.sourceEntity);
    if (data.id == "podcrash:open_furniture1_shop" && data.sourceEntity != null) openFurnitureShop1(data.sourceEntity);
    if (data.id == "podcrash:open_furniture2_shop" && data.sourceEntity != null) openFurnitureShop2(data.sourceEntity);
    if (data.id == "podcrash:open_decoration1_shop" && data.sourceEntity != null) openDecorationShop1(data.sourceEntity);
    if (data.id == "podcrash:open_decoration2_shop" && data.sourceEntity != null) openDecorationShop2(data.sourceEntity);
    if (data.id == "podcrash:open_festive_box_shop" && data.sourceEntity != null) openFestiveBoxShop(data.sourceEntity);
    if (data.id == "podcrash:open_festive_tree_shop" && data.sourceEntity != null) openFestiveTreeShop(data.sourceEntity);
    if (data.id == "podcrash:drop_item" && data.sourceEntity != null && data.message != undefined) dropItems(data.sourceEntity, data.message);
    if (data.id == "podcrash:spawn_raft" && data.sourceEntity != null) {
        if (overworld.getEntities({ tags: [data.sourceEntity.name] }).length == 0) {
            const raft = overworld.spawnEntity("podcrash:raft", { x: 284, y: -35, z: 728 });
            raft.addTag(data.sourceEntity.name);
            data.sourceEntity.runCommand(`ride @s start_riding @e[type=podcrash:raft,c=1,tag="${data.sourceEntity.name}"] teleport_rider`);
        } else {
            player.runCommand(`tellraw @s { "rawtext": [ { "translate": "feature.raft.already_spawned" } ] }`);
        }
    }
    if (data.id == "podcrash:display_navigation_text" && data.sourceEntity != null) displayText(data.sourceEntity);
    if (data.id == "podcrash:cut_wood" && data.sourceEntity != null) {
        const log = overworld.getEntities({type:"podcrash:log",location:data.sourceEntity.location,closest:1})[0];
        if(log===undefined) return;
        const state = log.getComponent("variant").value;
        if(state==4) {
            log.runCommand("event entity @s podcrash:despawn");
            log.runCommand("scoreboard players set log_cooldown value 0");
            giveCandy(data.sourceEntity, "1-5");
        } else {
            log.runCommand(`event entity @s podcrash:cut${state+1}`);
        }
    }

    
    // Light Show - Owen
    if (data.id == "podcrash:start_show") show.startShow();
    if (data.id == "podcrash:stop_show") show.stopShow();

});



system.runInterval(() => {
    runAvalanche();
});